import nest
import numpy as np
import sys
import os
import matplotlib.pyplot as plt
sys.path.insert(0, 'code/NEST_model/')
from params19Oct2023 import *

#------------------------------------------------
# FILE LABELS
label= '/home/nest/' #'/home/nest/data/NEST_model/'
memTestLabel='4memPotTest_'+pltLabel+'.png'
fmemTestOut=os.path.join(label, memTestLabel)
fmemPotIz1='4iz_membrane_potential_1_'+str(pltLabel)
fmemPotIz2='4iz_membrane_potential_2_'+str(pltLabel)
fmemPotIz3='4iz_membrane_potential_3_'+str(pltLabel)
fmemPotIz4='4iz_membrane_potential_4_'+str(pltLabel)
fspikesIZ1='4iz_spikes_1_'+str(pltLabel)
fspikesIZ2='4iz_spikes_2_'+str(pltLabel)
fspikesIZ3='4iz_spikes_3_'+str(pltLabel)
fspikesIZ4='4iz_spikes_4_'+str(pltLabel)


#-------------------------------------------
# KERNEL
nest.ResetKernel()
seed = [np.random.randint(0, 9999999)] * num_threads #nest.SetKernelStatus({'rng_seed': 12345})
nest.SetKernelStatus({'resolution': sim_resolution,  
                      'print_time': True,
                      'rng_seeds': seed,
                      'local_num_threads': num_threads, 
                      'overwrite_files': True})

#-------------------------------------------
# SPIKE GENERATORS: EXTERNAL STIM

s_ex1 = nest.Create("spike_generator", params={"spike_times": spTimes_exc1}) # Exc Generator #1
s_ex2 = nest.Create("spike_generator", params={"spike_times": spTimes_exc2}) # Exc Generator #2
s_in = nest.Create("spike_generator", params={"spike_times": spTimes_inh}) # Inh Generator #2


#-------------------------------------------
# NEURON POPs
n_iz2 = nest.Create("izhikevich", 4)
nest.SetStatus(n_iz2, params=exc_neuron_params)


#-------------------------------------------
# MULTIMETER
mm_iz1 = nest.Create("multimeter", params={
    "interval": sim_resolution,
    'record_from': ['V_m'], 
    'withgid': True, 
    'withtime': True, 
    'to_memory': True, 
    'precision': 17,  
    'to_file': True, 
    'label': os.path.join(label, fmemPotIz1)}) 

mm_iz2 = nest.Create("multimeter", params={
    "interval": sim_resolution,
    'record_from': ['V_m'],
    'withgid': True, 
    'withtime': True, 
    'to_memory': True, 
    'precision': 17,  
    'to_file': True, 
    'label': os.path.join(label, fmemPotIz2)}) 

mm_iz3 = nest.Create("multimeter", params={
    "interval": sim_resolution,
    'record_from': ['V_m'],
    'withgid': True, 
    'withtime': True, 
    'to_memory': True, 
    'precision': 17,  
    'to_file': True, 
    'label': os.path.join(label, fmemPotIz3)})  

mm_iz4 = nest.Create("multimeter", params={
    "interval": sim_resolution,
    'record_from': ['V_m'],
    'withgid': True, 
    'withtime': True, 
    'to_memory': True, 
    'precision': 17,  
    'to_file': True, 
    'label': os.path.join(label, fmemPotIz4)}) 


# SPIKEDETECTOR
spkDet_iz1 = nest.Create("spike_detector", params={
    'withgid': True, 
    'withtime': True,
    'to_memory': True,  
    'to_file': True, 
    'label': os.path.join(label, fspikesIZ1)})

spkDet_iz2 = nest.Create("spike_detector", params={
    'withgid': True, 
    'withtime': True,
    'to_memory': True,
    'to_file': True, 
    'label': os.path.join(label, fspikesIZ2)})

spkDet_iz3 = nest.Create("spike_detector", params={
    'withgid': True, 
    'withtime': True,
    'to_memory': True, 
    'to_file': True, 
    'label': os.path.join(label, fspikesIZ3)})

spkDet_iz4 = nest.Create("spike_detector", params={
    'withgid': True, 
    'withtime': True,
    'to_memory': True,
    'to_file': True,
    'label': os.path.join(label, fspikesIZ4)})


#-------------------------------------------
# CONNECTIONS

nest.Connect(s_ex1, [n_iz2[0]], syn_spec={"weight": W_exc})
nest.Connect(s_ex2, [n_iz2[0]], syn_spec={"weight": W_exc})
nest.Connect(s_in, [n_iz2[0]], syn_spec={"weight": W_inh})

nest.Connect([n_iz2[0]], [n_iz2[1]], syn_spec={'weight': W_exc, 'delay': 100.0})
nest.Connect([n_iz2[0]], [n_iz2[2]], syn_spec={'weight': W_exc, 'delay': 100.0})
nest.Connect([n_iz2[1]], [n_iz2[3]], syn_spec={'weight': W_exc, 'delay': 100.0})
nest.Connect([n_iz2[2]], [n_iz2[3]], syn_spec={'weight': W_exc, 'delay': 100.0})


nest.Connect(mm_iz1, n_iz2[0]) 
nest.Connect(mm_iz2, n_iz2[1]) 
nest.Connect(mm_iz3, n_iz2[2]) 
nest.Connect(mm_iz4, n_iz2[3]) 

nest.Connect(n_iz2[0], spkDet_iz1) 
nest.Connect(n_iz2[1], spkDet_iz2)
nest.Connect(n_iz2[2], spkDet_iz3)
nest.Connect(n_iz2[3], spkDet_iz4)



#-------------------------------------------
# SIMULATE
nest.Simulate(sim_time)



#--------------------
# PREP PLOTS

#IZ2 Membrane Potential
#maxPotIz=max(nest.GetStatus(mm_iz1)[0]["events"]["V_m"],nest.GetStatus(mm_iz2)[0]["events"]["V_m"],nest.GetStatus(mm_iz3)[0]["events"]["V_m"],nest.GetStatus(mm_iz4)[0]["events"]["V_m"]) 
#minPotIz=min(nest.GetStatus(mm_iz1)[0]["events"]["V_m"],nest.GetStatus(mm_iz2)[0]["events"]["V_m"],nest.GetStatus(mm_iz3)[0]["events"]["V_m"],nest.GetStatus(mm_iz4)[0]["events"]["V_m"])


#--------------------
# PLOTS

#fig3, axs3 = plt.subplots(4, 1, sharex=True)

#axs3[0].plot(nest.GetStatus(mm_iz1)[0]["events"]["times"],nest.GetStatus(mm_iz1)[0]["events"]["V_m"],'b-')
#axs3[0].set_xlim([0, 1000])
#axs3[0].set_ylim([int(minPotIz2-5.), int(maxPotIz2+8.)])
#axs3[1].plot(nest.GetStatus(mm_iz2)[0]["events"]["times"],nest.GetStatus(mm_iz2)[0]["events"]["V_m"],'r-')
#axs3[1].set_ylim([int(minPotIz-5.), int(maxPotIz+8.)])
#axs3[2].plot(nest.GetStatus(mm_iz3)[0]["events"]["times"],nest.GetStatus(mm_iz3)[0]["events"]["V_m"],'c-')
#axs3[2].set_ylim([int(minPotIz-5.), int(maxPotIz+8.)])
#axs3[3].plot(nest.GetStatus(mm_iz4)[0]["events"]["times"],nest.GetStatus(mm_iz4)[0]["events"]["V_m"],'g-')
#axs3[3].set_ylim([int(minPotIz-5.), int(maxPotIz+8.)])

#plot()
